import { createFragmentKey } from './fragment';
var Types = /** @class */ (function () {
    function Types() {
    }
    /**
     * Set the real server property name.
     * It means current property name is an alias.
     */
    Types.prototype.aliasOf = function (realName) {
        var that = this.clone();
        that.realName = realName;
        return that;
    };
    Object.defineProperty(Types.prototype, "number", {
        /**
         * Property is a number
         */
        get: function () {
            return this;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Types.prototype, "string", {
        /**
         * Property is a string
         */
        get: function () {
            return this;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Types.prototype, "boolean", {
        /**
         * Property is a bool
         */
        get: function () {
            return this;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Types.prototype, "undefined", {
        /**
         * Property may be undefined
         */
        get: function () {
            return this;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Types.prototype, "null", {
        /**
        * Property may be null
        */
        get: function () {
            return this;
        },
        enumerable: false,
        configurable: true
    });
    /**
    * Define custom type:
    * ```typescript
    * types.custom<'User'>();
    *
    * enum Gender {
    *   man = 'man',
    *   woman = 'woman',
    * }
    *
    * types.custom<Gender>();
    * ```
    */
    Types.prototype.custom = function () {
        return this;
    };
    /**
     * ```
     * {
     *   lists {
     *     id
     *     number
     *   }
     * }
     * ```
     */
    Types.prototype.object = function (items) {
        var that = this.clone();
        that.returns = items;
        return that;
    };
    /**
     * ```
     * {
     *   lists {
     *     id
     *     number
     *   }
     * }
     * ```
     */
    Types.prototype.array = function (
    // @ts-ignore
    each) {
        var that = this.clone();
        that.returns = each;
        return that;
    };
    /**
     * ```
     * {
     *   id @include (if: $varA)
     *   name
     *   lists (page: 1) @include (if: $varB) {
     *     id
     *     name
     *     age @include (if: $varC)
     *   }
     * }
     * ```
     */
    Types.prototype.include = function (param_Boolean) {
        var that = this.clone();
        that.includeParam = param_Boolean;
        that.appendParams([param_Boolean]);
        return that;
    };
    /**
     * ```
     * {
     *   id @skip (if: $varA)
     *   name
     *   lists (page: 1) @skip (if: $varB) {
     *     id
     *     name
     *     age @skip (if: $varC)
     *   }
     * }
     * ```
     */
    Types.prototype.skip = function (param_Boolean) {
        var that = this.clone();
        that.skipParam = param_Boolean;
        that.appendParams([param_Boolean]);
        return that;
    };
    /**
     *
     * @param {String[]} params_Type
     * @example `page: Int` | `name: String!` | `focus: Boolean` | `data: MyObject`
     * @param {Types} returns
     */
    Types.prototype.fn = function (params_Type, returns) {
        var that = this.clone();
        that.fnParams = params_Type;
        that.returns = returns;
        that.appendParams(params_Type);
        return that;
    };
    /**
     * Inline fragment with same struct
     * ```
     * lists {
     *   ... on User {
     *     name
     *   }
     *   ... on Admin {
     *     name
     *   }
     * }
     * ```
     */
    Types.prototype.on = function (on, body) {
        var data = {};
        var names = typeof on === 'string' ? [on] : on;
        for (var i = 0; i < names.length; ++i) {
            data[createFragmentKey()] = {
                name: '',
                on: names[i],
                inline: true,
                template: body,
                includeParam: this.includeParam,
                skipParam: this.skipParam,
            };
        }
        // @ts-ignore
        return data;
    };
    Types.prototype.union = function () {
        var data = {};
        Array.prototype.forEach.call(arguments, function (item) {
            Object.keys(item).forEach(function (key) {
                data[key] = item[key];
            });
        });
        return data;
    };
    Types.prototype.appendParams = function (names) {
        this.totalParams = this.totalParams || [];
        this.totalParams.push.apply(this.totalParams, names);
    };
    Types.prototype.clone = function () {
        // @ts-ignore
        var that = new this.constructor();
        if (this.fnParams) {
            that.fnParams = this.fnParams.slice();
        }
        if (this.totalParams) {
            that.totalParams = this.totalParams.slice();
        }
        this.includeParam && (that.includeParam = this.includeParam);
        this.skipParam && (that.skipParam = this.skipParam);
        this.realName && (that.realName = this.realName);
        this.returns && (that.returns = this.returns);
        // @ts-ignore
        return that;
    };
    return Types;
}());
export { Types };
export var types = new Types();
