"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = void 0;
var fragment_1 = require("./fragment");
var parameter_1 = require("./parameter");
var render_1 = require("./render");
var types_1 = require("./types");
var parse = function (type, name, nodes, ctx) {
    if (!name) {
        var firstName = Object.keys(nodes)[0];
        name = firstName.substr(0, 1).toUpperCase() + firstName.substr(1);
    }
    var body = cycleParse(nodes, ctx, -2);
    var params = ctx.params.map(function (key) {
        var param = parameter_1.parseParameter(key);
        return "$" + param.variable + ": " + param.type;
    });
    var paramStr = params.length ? " (" + params.join(', ') + ")" : '';
    return type + " " + name + paramStr + body + ctx.fragmentStrs.join('');
};
exports.parse = parse;
var cycleParse = function (nodes, ctx, space) {
    if (nodes instanceof types_1.Types) {
        parameter_1.collectParams(nodes.totalParams, ctx);
        var realName = nodes.realName ? ": " + nodes.realName : '';
        var directives = render_1.renderDirectives(nodes.includeParam, nodes.skipParam);
        // function
        if (nodes.fnParams) {
            var params = nodes.fnParams.map(function (key) {
                var param = parameter_1.parseParameter(key);
                return param.name + ": $" + param.variable;
            }).join(', ');
            return realName + " (" + params + ")" + directives + cycleParse(nodes.returns, ctx, space);
        }
        // Object or Array
        if (nodes.returns) {
            return "" + realName + directives + cycleParse(nodes.returns, ctx, space);
        }
        // string, number or boolean
        return realName + directives;
    }
    var stringNodes = {};
    // object
    Object.keys(nodes).forEach(function (key) {
        var node = nodes[key];
        // fragment
        if (fragment_1.isFragment(key)) {
            // @ts-ignore
            var fragment = node;
            var directives = render_1.renderDirectives(fragment.includeParam, fragment.skipParam);
            parameter_1.collectParams([fragment.includeParam, fragment.skipParam], ctx);
            if (fragment.inline) {
                stringNodes[key] = "... on " + fragment.on + directives + cycleParse(fragment.template, ctx, space + 2);
            }
            else {
                parseFragment(fragment, ctx);
                stringNodes[key] = "..." + fragment.tmpName + directives;
            }
        }
        else {
            stringNodes[key] = cycleParse(node, ctx, space + 2);
        }
    });
    return render_1.renderObject(stringNodes, space + 2);
};
var parseFragment = function (fragment, ctx) {
    var fragmentObjs = ctx.fragmentObjs;
    if (~fragmentObjs.indexOf(fragment)) {
        return;
    }
    var on = fragment.on, template = fragment.template, name = fragment.name;
    var indexes = ctx.fragmentIndex;
    indexes[on] = indexes[on] === undefined ? 0 : indexes[on] + 1;
    var tmpName = fragment.tmpName = name || on + "Fragment" + (indexes[on] === 0 ? '' : "_" + indexes[on]);
    fragmentObjs.push(fragment);
    ctx.fragmentStrs.push("\n\nfragment " + tmpName + " on " + on + cycleParse(template, ctx, -2));
};
